// print architecture, compiler, operating system and sizes in byes of the C++ builtin types
// @author (C) Copyright 2010 Paolo Greppi libpf.com.

// no guarranties whatsoever
// distribute freely and free of charge citing this:
// https://libpf.com/posts/2010-08-19-the-sizes-of-the-built-in-c-types-an-experimental-investigation

// architecture / compiler / operating system detection
// courtesy the "Pre-defined C/C++ Compiler Macros"
// http://predef.sourceforge.net

#include <iostream>

int main(int, char **) {

  bool arch(false);
  bool compiler(false);
  bool os(false);
  
  std::cout << "Architecture: ";
#if defined(__amd64__) || defined(_M_X64)
  std::cout << "AMD64" << std::endl; arch = true;
#endif
#if defined(__alpha__) || defined(__alpha) || defined(_M_ALPHA)
  std::cout << "ALPHA" << std::endl; arch = true;
#endif
#if defined(__arm__) || defined(__thumb__) || defined(__TARGET_ARCH_ARM) || defined(_ARM)
  std::cout << "ARM" << std::endl; arch = true;
#endif
#if defined(__hppa__) || defined(__hppa)
  std::cout << "HP/PA RISC" << std::endl; arch = true;
#endif
#if defined(__i386__) || defined(__i386) || defined(_M_IX86) || defined(__X86__) || defined(_X86_) || defined(__THW_INTEL__) || defined(__I86__) || defined(__INTEL__)
  std::cout << "Intel x86" << std::endl; arch = true;
#endif
#if defined(__ia64__) || defined(__ia64) || defined(_M_IA64)
  std::cout << "Intel Architecture-64" << std::endl; arch = true;
#endif
#if defined(__m68k__) || defined(M68000)
  std::cout << "Motorola 68k" << std::endl; arch = true;
#endif
#if defined(__mips__) || defined(__mips) || defined(__MIPS__)
  std::cout << "MIPS" << std::endl; arch = true;
#endif
#if defined(__powerpc__) || defined(_M_PPC) || defined(_ARCH_PPC)
  std::cout << "PowerPC" << std::endl; arch = true;
#endif
#if defined(__THW_RS6000) || defined(_IBMR2) || defined(_POWER) || defined(_ARCH_PWR)
  std::cout << "RS/6000" << std::endl; arch = true;
#endif
#if defined(__sparc__) || defined(__sparc)
  std::cout << "SPARC" << std::endl; arch = true;
#endif
#if defined(__sh__)
  std::cout << "SuperH" << std::endl; arch = true;
#endif
#if defined(__370__)
  std::cout << "System/370" << std::endl; arch = true;
#endif
#if defined(__s390__)
  std::cout << "System/390" << std::endl; arch = true;
#endif
#if defined(__SYSC_ZARCH__)
  std::cout << "z/Architecture" << std::endl; arch = true;
#endif
  if (!arch)
    std::cout << "UNRECOGNIZED" << std::endl;
  
  std::cout << "Compiler: ";
#if defined(__BORLANDC__)
  std::cout << "Borland C/C++" << std::endl; compiler = true;
#endif
#if defined(__COMO__)
  std::cout << "Comeau C++" << std::endl; compiler = true;
#endif
#if defined(_CRAYC)
  std::cout << "Cray C/C++" << std::endl; compiler = true;
#endif
#if defined(__CYGWIN__)
  std::cout << "Cygwin" << std::endl; compiler = true;
#endif
#if defined(__DECC) || defined(__DECCXX)
  std::cout << "Compaq C++" << std::endl; compiler = true;
#define PREDEF_COMPILER_DECC
#else
# if defined(VAXC) || defined(__VAXC)
  std::cout << "Compaq C++" << std::endl; compiler = true;
#define PREDEF_COMPILER_DECC
# else
#  if defined(__osf__) && defined(__LANGUAGE_C__) && !defined(__GNUC__)
/* Workaround for old DEC C compilers */
  std::cout << "Compaq C++" << std::endl; compiler = true;
#define PREDEF_COMPILER_DECC
#  endif
# endif
#endif
#if defined(__DCC__) && defined(__VERSION_NUMBER__)
  std::cout << "Diab C/C++" << std::endl; compiler = true;
#endif
#if defined(__DMC__)
  std::cout << "Digital Mars (Symatec C++, Zortech C++)" << std::endl; compiler = true;
#else
# if defined(__SC__) || defined(__ZTC__)
  std::cout << "Digital Mars (Symatec C++, Zortech C++)" << std::endl; compiler = true;
# endif
#endif
#if defined(__GNUC__)
  std::cout << "GNU C/C++" << std::endl; compiler = true;
#endif
#if defined(__HP_aCC)
  std::cout << "HP ANSI C / aC++" << std::endl; compiler = true;
#else
# if defined(__HP_cc)
  std::cout << "HP ANSI C / aC++" << std::endl; compiler = true;
# else
#  if (__cplusplus - 0 >= 199707L) && defined(__hpux)
  std::cout << "HP ANSI C / aC++" << std::endl; compiler = true;
#  endif
# endif
#endif
#if defined(__INTEL_COMPILER)
  std::cout << "Intel C++" << std::endl; compiler = true;
#else
# if defined(__ICC)
  std::cout << "Intel C++" << std::endl; compiler = true;
# endif
#endif
#if defined(__KCC)
  std::cout << "KAI C++" << std::endl; compiler = true;
#endif
#if defined(__MINGW32__)
  std::cout << "MinGW32+" << std::endl; compiler = true;
#endif
#if (defined(sgi) || defined(__sgi)) && defined(_COMPILER_VERSION)
  std::cout << "SGI MIPSpro" << std::endl; compiler = true;
#endif
#if defined(__MRC__)
  std::cout << "Apple MPW C++" << std::endl; compiler = true;
#else
# if defined(MPW_C) || defined(MPW_CPLUS)
  std::cout << "Apple MPW C++" << std::endl; compiler = true;
# endif
#endif
#if defined(__MWERKS__)
  std::cout << "Metrowerks CodeWarrior" << std::endl; compiler = true;
#endif
#if defined(__CC_NORCROFT)
  std::cout << "Norcroft C" << std::endl; compiler = true;
#endif
#if defined(_SCO_DS)
  std::cout << "SCO OpenServer" << std::endl; compiler = true;
#endif
#if defined(__SUNPRO_C)
  std::cout << "Sun Forte C (Workshop Pro)" << std::endl; compiler = true;
#define PREDEF_COMPILER_SUNPRO
#else
# if defined(__SUNPRO_CC)
  std::cout << "Sun Forte C/C++ (Workshop Pro)" << std::endl; compiler = true;
#define PREDEF_COMPILER_SUNPRO
# endif
#endif
#if defined(__TenDRA__)
  std::cout << "TenDRA" << std::endl; compiler = true;
#endif
#if defined(__USLC__)
  std::cout << "USL C" << std::endl; compiler = true;
#endif
#if defined(_MSC_VER)
  std::cout << "Microsoft Visual C++" << std::endl; compiler = true;
#endif
#if defined(__WATCOMC__)
  std::cout << "Watcom C++" << std::endl; compiler = true;
#endif
#if defined(__xlC__)
# define PREDEF_COMPILER_XLC
  std::cout << "IBM xlC" << std::endl; compiler = true;
#else
# if defined(__IBMC__) || defined(__IBMCPP__)
# define PREDEF_COMPILER_XLC
  std::cout << "IBM xlC" << std::endl; compiler = true;
# else
#  if defined(_AIX) && !defined(__GNUC__)
/* Workaround for old xlc */
# define PREDEF_COMPILER_XLC
  std::cout << "IBM xlC" << std::endl; compiler = true;
#  endif
# endif
#endif
  if (!compiler)
    std::cout << "UNRECOGNIZED" << std::endl;
  
  std::cout << "Operating System: ";
#if defined(__amigaos__) || defined(AMIGA)
  std::cout << "AmigaOS" << std::endl; os = true;
#endif
#if defined(_AIX) || defined(__TOS_AIX__) || defined(PREDEF_COMPILER_XLC)
  std::cout << "AIX" << std::endl; os = true;
#endif
#if defined(__BEOS__)
  std::cout << "BeOS" << std::endl; os = true;
#endif
#if defined(bsdi) || defined(__bsdi__)
  std::cout << "BSD/OS" << std::endl; os = true;
#endif
#if defined(_UNICOS) || defined(_CRAY)
  std::cout << "Cray UniCOS" << std::endl; os = true;
#endif
#if defined(DGUX) || defined(__DGUX__) || defined(__dgux__)
  std::cout << "DG/UX" << std::endl; os = true;
#endif
#if defined(__DragonFly__)
  std::cout << "DragonFly" << std::endl; os = true;
#endif
#if defined(__FreeBSD__)
  std::cout << "FreeBSD" << std::endl; os = true;
#endif
#if defined(hpux) || defined(__hpux) || defined(_HPUX_SOURCE)
  std::cout << "HP-UX" << std::endl; os = true;
#endif
#if defined(__GNU__)
  std::cout << "GNU Hurd" << std::endl; os = true;
#endif
#if defined(sgi) || defined(__sgi) || defined(mips) || defined(_SGI_SOURCE)
  std::cout << "IRIX" << std::endl; os = true;
#endif
#if defined(linux) || defined(__linux) || defined(__linux__)
  std::cout << "Linux" << std::endl; os = true;
#endif
#if defined(macintosh) || defined(Macintosh) || defined(__MACOS__)
  std::cout << "MacOS" << std::endl; os = true;
#endif
#if defined(__MACOSX__) || (defined(__APPLE__) && defined(__MACH__))
  std::cout << "MacOSX" << std::endl; os = true;
#endif
#if defined(mpeix) || defined(__mpexl)
  std::cout << "MPE/iX" << std::endl; os = true;
#endif
#if defined(MSDOS) || defined(__MSDOS__) || defined(_MSDOS) || defined(__DOS__)
  std::cout << "MSDOS" << std::endl; os = true;
#endif
#if defined(__NetBSD__)
  std::cout << "NetBSD" << std::endl; os = true;
#endif
#if defined(__OpenBSD__)
  std::cout << "OpenBSD" << std::endl; os = true;
#endif
#if defined(OS2) || defined(_OS2) || defined(__OS2__) || defined(__TOS_OS2__)
  std::cout << "OS/2" << std::endl; os = true;
#endif
#if defined(__osf__) || defined(__osf) || defined(PREDEF_COMPILER_DECC)
  std::cout << "Tru64 (OSF/1)" << std::endl; os = true;
#endif
#if defined(VMS) || defined(  __VMS)
  std::cout << "VMS" << std::endl; os = true;
#endif
#if defined(pyr)
  std::cout << "Pyramid DC/OS" << std::endl; os = true;
#endif
#if defined(__QNX__) || defined(__QNXNTO__)
  std::cout << "QNX" << std::endl; os = true;
#endif
#if defined(M_I386) || defined(M_XENIX) || defined(_SCO_C_DIALECT) || defined(_SCO_DS)
  std::cout << "SCO OpenServer" << std::endl; os = true;
#endif
#if defined(_SEQUENT_) || defined(sequent)
  std::cout << "DYNIX/ptx" << std::endl; os = true;
#endif
#if defined(sinix)
  std::cout << "Sinix" << std::endl; os = true;
#endif
#if defined(sun) || defined(__sun__) || defined(PREDEF_COMPILER_SUNPRO)
# if defined(__SVR4) || defined(__svr4__)
  std::cout << "Solaris" << std::endl; os = true;
# else
  std::cout << "Sun OS" << std::endl; os = true;
# endif
#endif
#if defined(ultrix) || defined(__ultrix) || defined(__ultrix__)
  std::cout << "Ultrix" << std::endl; os = true;
#endif
#if defined(__SYMBIAN32__)
  std::cout << "Symbian OS" << std::endl; os = true;
#endif
#if defined(WIN32) || defined(_WIN32) || defined(__TOS_WIN__) || defined(_MSC_VER)
  std::cout << "Windows" << std::endl; os = true;
#endif

#if defined(__MVS__) || defined(__HOS_MVS__)
  std::cout << "z/OS (OS/390)" << std::endl; os = true;
#endif
#if defined(__IPHONE_2_0) || defined(__IPHONE_2_1) || defined(__IPHONE_2_2) || defined(__IPHONE_3_0) || defined(__IPHONE_NA)
  std::cout << "iPhone OS" << std::endl; os = true;
#endif
#if defined(sinix)
  std::cout << "Sinix" << std::endl; os = true;
#endif

  if (!os)
    std::cout << "UNRECOGNIZED" << std::endl;


  std::cout << std::endl;

  std::cout << "sizeof(void *) = " << sizeof(void *) << std::endl;
  std::cout << std::endl;
  
  std::cout << "sizeof(short int) = " << sizeof(short int) << std::endl;
  std::cout << "sizeof(int) = " << sizeof(int) << std::endl;
  std::cout << "sizeof(long int) = " << sizeof(long int) << std::endl;
  std::cout << "sizeof(long long int) = " << sizeof(long long int) << std::endl;
  std::cout << std::endl;
  
  std::cout << "sizeof(float) = " << sizeof(float) << std::endl;
  std::cout << "sizeof(double) = " << sizeof(double) << std::endl;
  std::cout << "sizeof(long double) = " << sizeof(long double) << std::endl;
  std::cout << std::endl;
  
  std::cout << "sizeof(char) = " << sizeof(char) << std::endl;
  std::cout << "sizeof(wchar_t) = " << sizeof(wchar_t) << std::endl;
  std::cout << std::endl;
  
  std::cout << "sizeof(bool) = " << sizeof(bool) << std::endl;
  std::cout << std::endl;

} // main
